@@ -25,18 +25,13 @@
  * SPDX-License-Identifier: MIT
  */
 
-#include "ares_setup.h"
+#include "ares_private.h"
 
 #ifdef HAVE_NETINET_IN_H
 #  include <netinet/in.h>
 #endif
-
 #include "ares_nameser.h"
 
-#include "ares.h"
-#include "ares_dns.h"
-#include "ares_private.h"
-
 static unsigned short generate_unique_qid(ares_channel_t *channel)
 {
   unsigned short id;
@@ -48,18 +43,77 @@ static unsigned short generate_unique_qid(ares_channel_t *channel)
   return id;
 }
 
-static ares_status_t ares_send_dnsrec_int(ares_channel_t          *channel,
-                                          const ares_dns_record_t *dnsrec,
-                                          ares_callback_dnsrec     callback,
-                                          void *arg, unsigned short *qid)
+/* https://datatracker.ietf.org/doc/html/draft-vixie-dnsext-dns0x20-00 */
+static ares_status_t ares_apply_dns0x20(ares_channel_t    *channel,
+                                        ares_dns_record_t *dnsrec)
 {
-  struct query            *query;
-  size_t                   packetsz;
-  ares_timeval_t           now = ares__tvnow();
+  ares_status_t status = ARES_SUCCESS;
+  const char   *name   = NULL;
+  char          dns0x20name[256];
+  unsigned char randdata[256 / 8];
+  size_t        len;
+  size_t        remaining_bits;
+  size_t        total_bits;
+  size_t        i;
+
+  status = ares_dns_record_query_get(dnsrec, 0, &name, NULL, NULL);
+  if (status != ARES_SUCCESS) {
+    goto done;
+  }
+
+  len = ares_strlen(name);
+  if (len == 0 || len >= sizeof(dns0x20name)) {
+    status = ARES_EBADNAME;
+    goto done;
+  }
+
+  memset(dns0x20name, 0, sizeof(dns0x20name));
+
+  /* Fetch the minimum amount of random data we'd need for the string, which
+   * is 1 bit per byte */
+  total_bits     = ((len + 7) / 8) * 8;
+  remaining_bits = total_bits;
+  ares__rand_bytes(channel->rand_state, randdata, total_bits / 8);
+
+  /* Randomly apply 0x20 to name */
+  for (i = 0; i < len; i++) {
+    size_t bit;
+
+    /* Only apply 0x20 to alpha characters */
+    if (!ares__isalpha(name[i])) {
+      dns0x20name[i] = name[i];
+      continue;
+    }
+
+    /* coin flip */
+    bit = total_bits - remaining_bits;
+    if (randdata[bit / 8] & (1 << (bit % 8))) {
+      dns0x20name[i] = name[i] | 0x20;                          /* Set 0x20 */
+    } else {
+      dns0x20name[i] = (char)(((unsigned char)name[i]) & 0xDF); /* Unset 0x20 */
+    }
+    remaining_bits--;
+  }
+
+  status = ares_dns_record_query_set_name(dnsrec, 0, dns0x20name);
+
+done:
+  return status;
+}
+
+ares_status_t ares_send_nolock(ares_channel_t          *channel,
+                               const ares_dns_record_t *dnsrec,
+                               ares_callback_dnsrec callback, void *arg,
+                               unsigned short *qid)
+{
+  ares_query_t            *query;
+  ares_timeval_t           now;
   ares_status_t            status;
   unsigned short           id          = generate_unique_qid(channel);
   const ares_dns_record_t *dnsrec_resp = NULL;
 
+  ares__tvnow(&now);
+
   if (ares__slist_len(channel->servers) == 0) {
     callback(arg, ARES_ENOSERVER, 0, NULL);
     return ARES_ENOSERVER;
@@ -75,29 +129,40 @@ static ares_status_t ares_send_dnsrec_int(ares_channel_t          *channel,
   }
 
   /* Allocate space for query and allocated fields. */
-  query = ares_malloc(sizeof(struct query));
+  query = ares_malloc(sizeof(ares_query_t));
   if (!query) {
-    callback(arg, ARES_ENOMEM, 0, NULL);
-    return ARES_ENOMEM;
+    callback(arg, ARES_ENOMEM, 0, NULL); /* LCOV_EXCL_LINE: OutOfMemory */
+    return ARES_ENOMEM;                  /* LCOV_EXCL_LINE: OutOfMemory */
   }
   memset(query, 0, sizeof(*query));
 
-  query->channel = channel;
+  query->channel      = channel;
+  query->qid          = id;
+  query->timeout.sec  = 0;
+  query->timeout.usec = 0;
+  query->using_tcp =
+    (channel->flags & ARES_FLAG_USEVC) ? ARES_TRUE : ARES_FALSE;
 
-  status = ares_dns_write(dnsrec, &query->qbuf, &query->qlen);
+  /* Duplicate Query */
+  status = ares_dns_record_duplicate_ex(&query->query, dnsrec);
   if (status != ARES_SUCCESS) {
     ares_free(query);
     callback(arg, status, 0, NULL);
     return status;
   }
 
-  query->qid          = id;
-  query->timeout.sec  = 0;
-  query->timeout.usec = 0;
+  ares_dns_record_set_id(query->query, id);
 
-  /* Ignore first 2 bytes, assign our own query id */
-  query->qbuf[0] = (unsigned char)((id >> 8) & 0xFF);
-  query->qbuf[1] = (unsigned char)(id & 0xFF);
+  if (channel->flags & ARES_FLAG_DNS0x20 && !query->using_tcp) {
+    status = ares_apply_dns0x20(channel, query->query);
+    if (status != ARES_SUCCESS) {
+      /* LCOV_EXCL_START: OutOfMemory */
+      callback(arg, status, 0, NULL);
+      ares__free_query(query);
+      return status;
+      /* LCOV_EXCL_STOP */
+    }
+  }
 
   /* Fill in query arguments. */
   query->callback = callback;
@@ -106,9 +171,6 @@ static ares_status_t ares_send_dnsrec_int(ares_channel_t          *channel,
   /* Initialize query status. */
   query->try_count = 0;
 
-  packetsz = (channel->flags & ARES_FLAG_EDNS) ? channel->ednspsz : PACKETSZ;
-  query->using_tcp =
-    (channel->flags & ARES_FLAG_USEVC) || query->qlen > packetsz;
 
   query->error_status = ARES_SUCCESS;
   query->timeouts     = 0;
@@ -121,18 +183,22 @@ static ares_status_t ares_send_dnsrec_int(ares_channel_t          *channel,
   query->node_all_queries =
     ares__llist_insert_last(channel->all_queries, query);
   if (query->node_all_queries == NULL) {
+    /* LCOV_EXCL_START: OutOfMemory */
     callback(arg, ARES_ENOMEM, 0, NULL);
     ares__free_query(query);
     return ARES_ENOMEM;
+    /* LCOV_EXCL_STOP */
   }
 
   /* Keep track of queries bucketed by qid, so we can process DNS
    * responses quickly.
    */
   if (!ares__htable_szvp_insert(channel->queries_by_qid, query->qid, query)) {
+    /* LCOV_EXCL_START: OutOfMemory */
     callback(arg, ARES_ENOMEM, 0, NULL);
     ares__free_query(query);
     return ARES_ENOMEM;
+    /* LCOV_EXCL_STOP */
   }
 
   /* Perform the first query action. */
@@ -152,12 +218,12 @@ ares_status_t ares_send_dnsrec(ares_channel_t          *channel,
   ares_status_t status;
 
   if (channel == NULL) {
-    return ARES_EFORMERR;
+    return ARES_EFORMERR; /* LCOV_EXCL_LINE: DefensiveCoding */
   }
 
   ares__channel_lock(channel);
 
-  status = ares_send_dnsrec_int(channel, dnsrec, callback, arg, qid);
+  status = ares_send_nolock(channel, dnsrec, callback, arg, qid);
 
   ares__channel_unlock(channel);
 
@@ -189,10 +255,12 @@ void ares_send(ares_channel_t *channel, const unsigned char *qbuf, int qlen,
 
   carg = ares__dnsrec_convert_arg(callback, arg);
   if (carg == NULL) {
+    /* LCOV_EXCL_START: OutOfMemory */
     status = ARES_ENOMEM;
     ares_dns_record_destroy(dnsrec);
     callback(arg, (int)status, 0, NULL, 0);
     return;
+    /* LCOV_EXCL_STOP */
   }
 
   ares_send_dnsrec(channel, dnsrec, ares__dnsrec_convert_cb, carg, NULL);
