@@ -876,6 +876,8 @@ TEST_P(MockChannelTest, SearchDomainsBare) {
   ares_gethostbyname(channel_, "www", AF_INET, HostCallback, &result);
   Process();
   EXPECT_TRUE(result.done_);
+  EXPECT_EQ(0, result.timeouts_);
+
   std::stringstream ss;
   ss << result.host_;
   EXPECT_EQ("{'www' aliases=[] addrs=[2.3.4.5]}", ss.str());
@@ -1005,7 +1007,9 @@ TEST_P(MockChannelTest, SearchHighNdots) {
 
 // Test that performing an EDNS search with an OPT RR options value works. The
 // options value should be included on the requests to the mock server.
-TEST_P(MockEDNSChannelTest, SearchOptVal) {
+// We are going to do this only via TCP since this won't include the dynamically
+// generated DNS cookie that would otherwise mess with this result.
+TEST_P(MockTCPChannelTest, SearchOptVal) {
   /* Define the OPT RR options code and value to use. */
   unsigned short opt_opt = 3;
   unsigned char opt_val[] = { 'c', '-', 'a', 'r', 'e', 's' };
@@ -1511,6 +1515,271 @@ TEST_P(MockChannelTest, GetHostByAddrDestroy) {
   EXPECT_EQ(0, result.timeouts_);
 }
 
+static const unsigned char *
+  fetch_server_cookie(const ares_dns_record_t *dnsrec, size_t *len)
+{
+  const ares_dns_rr_t *rr  = fetch_rr_opt(dnsrec);
+  const unsigned char *val = NULL;
+  *len                     = 0;
+
+  if (rr == NULL) {
+    return NULL;
+  }
+
+  if (!ares_dns_rr_get_opt_byid(rr, ARES_RR_OPT_OPTIONS, ARES_OPT_PARAM_COOKIE,
+                                &val, len)) {
+    return NULL;
+  }
+
+  if (*len <= 8) {
+    *len = 0;
+    return NULL;
+  }
+
+  *len -= 8;
+  val  += 8;
+  return val;
+}
+
+static const unsigned char *
+  fetch_client_cookie(const ares_dns_record_t *dnsrec, size_t *len)
+{
+  const ares_dns_rr_t *rr  = fetch_rr_opt(dnsrec);
+  const unsigned char *val = NULL;
+  *len                     = 0;
+
+  if (rr == NULL) {
+    return NULL;
+  }
+
+  if (!ares_dns_rr_get_opt_byid(rr, ARES_RR_OPT_OPTIONS, ARES_OPT_PARAM_COOKIE,
+                                &val, len)) {
+    return NULL;
+  }
+
+  if (*len < 8) {
+    *len = 0;
+    return NULL;
+  }
+
+  *len = 8;
+  return val;
+}
+
+TEST_P(MockUDPChannelTest, DNSCookieSingle) {
+  DNSPacket reply;
+  std::vector<byte> server_cookie = { 1, 2, 3, 4, 5, 6, 7, 8 };
+  reply.set_response().set_aa()
+    .add_question(new DNSQuestion("www.google.com", T_A))
+    .add_answer(new DNSARR("www.google.com", 0x0100, {0x01, 0x02, 0x03, 0x04}))
+    .add_additional(new DNSOptRR(0, 0, 0, 1280, { }, server_cookie, false));
+  EXPECT_CALL(server_, OnRequest("www.google.com", T_A))
+    .WillOnce(SetReply(&server_, &reply));
+
+  QueryResult result;
+  ares_query_dnsrec(channel_, "www.google.com", ARES_CLASS_IN, ARES_REC_TYPE_A, QueryCallback, &result, NULL);
+  Process();
+  EXPECT_TRUE(result.done_);
+  EXPECT_EQ(0, result.timeouts_);
+
+  size_t len;
+  const unsigned char *returned_cookie = fetch_server_cookie(result.dnsrec_.dnsrec_, &len);
+  EXPECT_EQ(len, server_cookie.size());
+  EXPECT_TRUE(memcmp(server_cookie.data(), returned_cookie, len) == 0);
+}
+
+TEST_P(MockUDPChannelTest, DNSCookieMissingAfterGood) {
+  std::vector<byte> server_cookie = { 1, 2, 3, 4, 5, 6, 7, 8 };
+  DNSPacket reply;
+  reply.set_response().set_aa()
+    .add_question(new DNSQuestion("www.google.com", T_A))
+    .add_answer(new DNSARR("www.google.com", 0x0100, {0x01, 0x02, 0x03, 0x04}))
+    .add_additional(new DNSOptRR(0, 0, 0, 1280, { }, server_cookie, false));
+  DNSPacket reply_nocookie;
+  reply_nocookie.set_response().set_aa()
+    .add_question(new DNSQuestion("www.google.com", T_A))
+    .add_answer(new DNSARR("www.google.com", 0x0100, {0x01, 0x02, 0x03, 0x04}))
+    .add_additional(new DNSOptRR(0, 0, 0, 1280, { }, { }, false));
+  DNSPacket reply_ensurecookie;
+  reply_ensurecookie.set_response().set_aa()
+    .add_question(new DNSQuestion("www.google.com", T_A))
+    .add_answer(new DNSARR("www.google.com", 0x0100, {0x01, 0x02, 0x03, 0x04}))
+    .add_additional(new DNSOptRR(0, 0, 0, 1280, { }, server_cookie, true));
+
+  EXPECT_CALL(server_, OnRequest("www.google.com", T_A))
+    .WillOnce(SetReply(&server_, &reply))
+    .WillOnce(SetReply(&server_, &reply_nocookie))
+    .WillOnce(SetReply(&server_, &reply_ensurecookie));
+
+  /* This test will establish the server supports cookies, then the next reply
+   * will be missing the server cookie and therefore be rejected and timeout, then
+   * an internal retry will occur and the cookie will be present again and it
+   * will be verified a server cookie was actually present that matches the
+   * server cookie. */
+  QueryResult result1;
+  ares_query_dnsrec(channel_, "www.google.com", ARES_CLASS_IN, ARES_REC_TYPE_A, QueryCallback, &result1, NULL);
+  Process();
+  EXPECT_TRUE(result1.done_);
+  EXPECT_EQ(0, result1.timeouts_);
+
+  QueryResult result2;
+  ares_query_dnsrec(channel_, "www.google.com", ARES_CLASS_IN, ARES_REC_TYPE_A, QueryCallback, &result2, NULL);
+  Process();
+  EXPECT_TRUE(result2.done_);
+  EXPECT_EQ(1, result2.timeouts_);
+
+  /* Client cookie should NOT have rotated */
+  size_t len1;
+  const unsigned char *client_cookie_1 = fetch_client_cookie(result1.dnsrec_.dnsrec_, &len1);
+  size_t len2;
+  const unsigned char *client_cookie_2 = fetch_client_cookie(result2.dnsrec_.dnsrec_, &len2);
+  EXPECT_EQ(len1, 8);
+  EXPECT_EQ(len1, len2);
+  EXPECT_TRUE(memcmp(client_cookie_1, client_cookie_2, len1) == 0);
+}
+
+
+TEST_P(MockUDPChannelTest, DNSCookieBadLen) {
+  std::vector<byte> server_cookie = { 1, 2, 3, 4, 5, 6, 7, 8 };
+  std::vector<byte> server_cookie_bad = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0xA, 0xB, 0xC, 0xD, 0xE, 0xF, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0xA, 0xB, 0xC, 0xD, 0xE, 0xF, 0 };
+  DNSPacket reply;
+  reply.set_response().set_aa()
+    .add_question(new DNSQuestion("www.google.com", T_A))
+    .add_answer(new DNSARR("www.google.com", 0x0100, {0x01, 0x02, 0x03, 0x04}))
+    .add_additional(new DNSOptRR(0, 0, 0, 1280, { }, server_cookie, false));
+  DNSPacket reply_badcookielen;
+  reply_badcookielen.set_response().set_aa()
+    .add_question(new DNSQuestion("www.google.com", T_A))
+    .add_answer(new DNSARR("www.google.com", 0x0100, {0x01, 0x02, 0x03, 0x04}))
+    .add_additional(new DNSOptRR(0, 0, 0, 1280, { }, server_cookie_bad, false));
+
+  EXPECT_CALL(server_, OnRequest("www.google.com", T_A))
+    .WillOnce(SetReply(&server_, &reply_badcookielen))
+    .WillOnce(SetReply(&server_, &reply));
+
+  /* This test will send back a malformed cookie len, then when it times out and retry occurs will send back a valid cookie. */
+  QueryResult result1;
+  ares_query_dnsrec(channel_, "www.google.com", ARES_CLASS_IN, ARES_REC_TYPE_A, QueryCallback, &result1, NULL);
+  Process();
+  EXPECT_TRUE(result1.done_);
+  EXPECT_EQ(1, result1.timeouts_);
+}
+
+
+TEST_P(MockUDPChannelTest, DNSCookieServerRotate) {
+  std::vector<byte> server_cookie = { 1, 2, 3, 4, 5, 6, 7, 8 };
+  std::vector<byte> server_cookie_rotate = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0xA, 0xB, 0xC, 0xD, 0xE, 0xF };
+
+  DNSPacket reply_cookie1;
+  reply_cookie1.set_response().set_aa()
+    .add_question(new DNSQuestion("www.google.com", T_A))
+    .add_answer(new DNSARR("www.google.com", 0x0100, {0x01, 0x02, 0x03, 0x04}))
+    .add_additional(new DNSOptRR(0, 0, 0, 1280, {}, server_cookie, false));
+  DNSPacket reply_cookie2_badcookie;
+  reply_cookie2_badcookie.set_response().set_aa().set_rcode(ARES_RCODE_BADCOOKIE & 0xF)
+    .add_question(new DNSQuestion("www.google.com", T_A))
+    .add_answer(new DNSARR("www.google.com", 0x0100, {0x01, 0x02, 0x03, 0x04}))
+    .add_additional(new DNSOptRR((ARES_RCODE_BADCOOKIE >> 4) & 0xFF, 0, 0, 1280, { }, server_cookie_rotate, false));
+  DNSPacket reply_cookie2;
+  reply_cookie2.set_response().set_aa()
+    .add_question(new DNSQuestion("www.google.com", T_A))
+    .add_answer(new DNSARR("www.google.com", 0x0100, {0x01, 0x02, 0x03, 0x04}))
+    .add_additional(new DNSOptRR(0, 0, 0, 1280, { }, server_cookie_rotate, true));
+
+  EXPECT_CALL(server_, OnRequest("www.google.com", T_A))
+    .WillOnce(SetReply(&server_, &reply_cookie1))
+    .WillOnce(SetReply(&server_, &reply_cookie2_badcookie))
+    .WillOnce(SetReply(&server_, &reply_cookie2));
+
+  /* This test will establish the server supports cookies, then the next reply
+   * the server returns BADCOOKIE indicating the cookie has rotated and
+   * returns a new cookie. Then the query will be automatically retried with
+   * the newly returned cookie. No timeouts should be indicated, and the
+   * client cookie should not rotate. */
+  QueryResult result1;
+  ares_query_dnsrec(channel_, "www.google.com", ARES_CLASS_IN, ARES_REC_TYPE_A, QueryCallback, &result1, NULL);
+  Process();
+  EXPECT_TRUE(result1.done_);
+  EXPECT_EQ(0, result1.timeouts_);
+
+  QueryResult result2;
+  ares_query_dnsrec(channel_, "www.google.com", ARES_CLASS_IN, ARES_REC_TYPE_A, QueryCallback, &result2, NULL);
+  Process();
+  EXPECT_TRUE(result2.done_);
+  EXPECT_EQ(0, result2.timeouts_);
+
+  /* Client cookie should NOT have rotated */
+  size_t len1;
+  const unsigned char *client_cookie_1 = fetch_client_cookie(result1.dnsrec_.dnsrec_, &len1);
+  size_t len2;
+  const unsigned char *client_cookie_2 = fetch_client_cookie(result2.dnsrec_.dnsrec_, &len2);
+  EXPECT_EQ(len1, 8);
+  EXPECT_EQ(len1, len2);
+  EXPECT_TRUE(memcmp(client_cookie_1, client_cookie_2, len1) == 0);
+}
+
+TEST_P(MockUDPChannelTest, DNSCookieSpoof) {
+  std::vector<byte> client_cookie = { 1, 2, 3, 4, 5, 6, 7, 8 };
+  std::vector<byte> server_cookie = { 1, 2, 3, 4, 5, 6, 7, 8 };
+
+  DNSPacket reply_spoof;
+  reply_spoof.set_response().set_aa()
+    .add_question(new DNSQuestion("www.google.com", T_A))
+    .add_answer(new DNSARR("www.google.com", 0x0100, {0x01, 0x02, 0x03, 0x04}))
+    .add_additional(new DNSOptRR(0, 0, 0, 1280, client_cookie, server_cookie, false));
+  DNSPacket reply;
+  reply.set_response().set_aa()
+    .add_question(new DNSQuestion("www.google.com", T_A))
+    .add_answer(new DNSARR("www.google.com", 0x0100, {0x01, 0x02, 0x03, 0x04}))
+    .add_additional(new DNSOptRR(0, 0, 0, 1280, { }, server_cookie, false));
+
+  EXPECT_CALL(server_, OnRequest("www.google.com", T_A))
+    .WillOnce(SetReply(&server_, &reply_spoof))
+    .WillOnce(SetReply(&server_, &reply));
+
+  /* This test will return a reply that doesn't have the same client cookie as
+   * was sent, this should result in a drop of the packet alltogether, then
+   * the library will retry and a proper result will be sent. */
+  QueryResult result;
+  ares_query_dnsrec(channel_, "www.google.com", ARES_CLASS_IN, ARES_REC_TYPE_A, QueryCallback, &result, NULL);
+  Process();
+  EXPECT_TRUE(result.done_);
+  EXPECT_EQ(1, result.timeouts_);
+}
+
+TEST_P(MockUDPChannelTest, DNSCookieTCPUpgrade) {
+  std::vector<byte> server_cookie = { 1, 2, 3, 4, 5, 6, 7, 8 };
+
+  DNSPacket reply_badcookie;
+  reply_badcookie.set_response().set_aa().set_rcode(ARES_RCODE_BADCOOKIE & 0xF)
+    .add_question(new DNSQuestion("www.google.com", T_A))
+    .add_answer(new DNSARR("www.google.com", 0x0100, {0x01, 0x02, 0x03, 0x04}))
+    .add_additional(new DNSOptRR((ARES_RCODE_BADCOOKIE >> 4) & 0xFF, 0, 0, 1280, { }, server_cookie, false));
+  DNSPacket reply;
+  reply.set_response().set_aa()
+    .add_question(new DNSQuestion("www.google.com", T_A))
+    .add_answer(new DNSARR("www.google.com", 0x0100, {0x01, 0x02, 0x03, 0x04}))
+    .add_additional(new DNSOptRR(0, 0, 0, 1280, { }, { }, false));
+
+  EXPECT_CALL(server_, OnRequest("www.google.com", T_A))
+    .WillOnce(SetReply(&server_, &reply_badcookie))
+    .WillOnce(SetReply(&server_, &reply_badcookie))
+    .WillOnce(SetReply(&server_, &reply_badcookie))
+    .WillOnce(SetReply(&server_, &reply));
+
+  /* This test will establish the server supports cookies, but continuously
+   * returns BADCOOKIE which is an indicator that there is some form of
+   * AnyCast issue across servers, so it upgrades to TCP afterwards.  No
+   * timeouts are recorded as the queries are sent back-to-back as immediate
+   * reattempts after the response. */
+  QueryResult result;
+  ares_query_dnsrec(channel_, "www.google.com", ARES_CLASS_IN, ARES_REC_TYPE_A, QueryCallback, &result, NULL);
+  Process();
+  EXPECT_TRUE(result.done_);
+  EXPECT_EQ(0, result.timeouts_);
+}
+
+
 #ifndef WIN32
 TEST_P(MockChannelTest, HostAlias) {
   DNSPacket reply;
@@ -1739,7 +2008,16 @@ TEST_P(NoRotateMultiMockTest, ServerNoResponseFailover) {
   EXPECT_EQ("{'www.example.com' aliases=[] addrs=[2.3.4.5]}", ss4.str());
 }
 
-#define SERVER_FAILOVER_RETRY_DELAY 750
+#ifdef __APPLE__
+/* Apple's sleep is no where near accurate, especially on a heavily loaded
+ * system.  Its possible we could set a realtime priority on a thread to
+ * help with this, but for now, just increase the delay */
+#  define SERVER_FAILOVER_RETRY_DELAY 750
+#elif defined(_WIN32)
+#  define SERVER_FAILOVER_RETRY_DELAY 500
+#else
+#  define SERVER_FAILOVER_RETRY_DELAY 250
+#endif
 class ServerFailoverOptsMultiMockTest : public MockMultiServerChannelTest {
  public:
   ServerFailoverOptsMultiMockTest()
